@@ -32,6 +32,7 @@ static PyObject* Binary;
 static PyObject* Code;
 static PyObject* ObjectId;
 static PyObject* DBRef;
+static PyObject* RECompile;
 
 static char* shuffle_oid(const char* oid) {
     char* shuffled = (char*) malloc(12);
@@ -52,28 +53,6 @@ static char* shuffle_oid(const char* oid) {
     return shuffled;
 }
 
-static PyObject* _cbson_shuffle_oid(PyObject* self, PyObject* args) {
-    char* data;
-    int length;
-    if (!PyArg_ParseTuple(args, "s#", &data, &length)) {
-        return NULL;
-    }
-
-    if (length != 12) {
-        PyErr_SetString(PyExc_ValueError, "oid must be of length 12");
-        return NULL;
-    }
-
-    char* shuffled = shuffle_oid(data);
-    if (!shuffled) {
-        return NULL;
-    }
-
-    PyObject* result = Py_BuildValue("s#", shuffled, 12);
-    free(shuffled);
-    return result;
-}
-
 static PyObject* build_element(const char type, const char* name, const int length, const char* data) {
     int name_length = strlen(name) + 1;
     int built_length = 1 + name_length + length;
@@ -380,13 +359,259 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* dict) {
     return NULL;
 }
 
+static PyObject* _elements_to_dict(PyObject* elements) {
+    PyObject* dict = PyDict_New();
+    if (!dict) {
+        return NULL;
+    }
+    const char* string = PyString_AsString(elements);
+    int max = PyString_Size(elements);
+    int position = 0;
+    while (position < max) {
+        int type = (int)string[position++];
+        int name_length = strlen(string + position);
+        PyObject* name = PyUnicode_DecodeUTF8(string + position, name_length, "strict");
+        if (!name) {
+            return NULL;
+        }
+        position += name_length + 1;
+        PyObject* value;
+        switch (type) {
+        case 1:
+            {
+                double d;
+                memcpy(&d, string + position, 8);
+                value = PyFloat_FromDouble(d);
+                if (!value) {
+                    return NULL;
+                }
+                position += 8;
+                break;
+            }
+        case 2:
+        case 13:
+        case 14:
+            {
+                position += 4;
+                int value_length = strlen(string + position);
+                value = PyUnicode_DecodeUTF8(string + position, value_length, "strict");
+                if (!value) {
+                    return NULL;
+                }
+                position += value_length + 1;
+                break;
+            }
+        case 3:
+            {
+                int size;
+                memcpy(&size, string + position, 4);
+                PyObject* array_elements = PyString_FromStringAndSize(string + position + 4, size - 5);
+                if (!array_elements) {
+                    return NULL;
+                }
+                value = _elements_to_dict(array_elements);
+                if (!value) {
+                    return NULL;
+                }
+                position += size;
+                break;
+            }
+        case 4:
+            {
+                int size;
+                memcpy(&size, string + position, 4);
+                PyObject* array_elements = PyString_FromStringAndSize(string + position + 4, size - 5);
+                if (!array_elements) {
+                    return NULL;
+                }
+                PyObject* array_dict = _elements_to_dict(array_elements);
+                if (!array_dict) {
+                    return NULL;
+                }
+                position += size;
+
+                value = PyList_New(0);
+                int length = PyDict_Size(array_dict);
+                int i;
+                for (i = 0; i < length; i++) {
+                    char* key;
+                    asprintf(&key, "%d", i);
+                    if (!key) {
+                        PyErr_NoMemory();
+                        return NULL;
+                    }
+                    PyObject* to_append = PyDict_GetItemString(array_dict, key);
+                    free(key);
+                    if (!to_append) {
+                        return NULL;
+                    }
+                    PyList_Append(value, to_append);
+                }
+                break;
+            }
+        case 5:
+            {
+                int length;
+                memcpy(&length, string + position, 4);
+                int subtype = (int)string[position + 4];
+                PyObject* data;
+                if (subtype == 2) {
+                    data = PyString_FromStringAndSize(string + position + 9, length - 4);
+                } else {
+                    data = PyString_FromStringAndSize(string + position + 5, length);
+                }
+                if (!data) {
+                    return NULL;
+                }
+                value = PyObject_CallFunctionObjArgs(Binary, data, PyInt_FromLong(subtype), NULL);
+                if (!value) {
+                    return NULL;
+                }
+                position += length + 5;
+                break;
+            }
+        case 6:
+        case 10:
+            {
+                value = Py_None;
+                break;
+            }
+        case 7:
+            {
+                char* shuffled = shuffle_oid(string + position);
+                value = PyObject_CallFunction(ObjectId, "s#", shuffled, 12);
+                free(shuffled);
+                if (!value) {
+                    return NULL;
+                }
+                position += 12;
+                break;
+            }
+        case 8:
+            {
+                value = string[position++] ? Py_True : Py_False;
+                break;
+            }
+        case 9:
+            {
+                long long millis;
+                memcpy(&millis, string + position, 8);
+                int microseconds = (millis % 1000) * 1000;
+                time_t seconds = millis / 1000;
+                struct tm* timeinfo = gmtime(&seconds);
+
+                value = PyDateTime_FromDateAndTime(timeinfo->tm_year + 1900,
+                                                   timeinfo->tm_mon + 1,
+                                                   timeinfo->tm_mday,
+                                                   timeinfo->tm_hour,
+                                                   timeinfo->tm_min,
+                                                   timeinfo->tm_sec,
+                                                   microseconds);
+                position += 8;
+                break;
+            }
+        case 11:
+            {
+                int pattern_length = strlen(string + position);
+                PyObject* pattern = PyUnicode_DecodeUTF8(string + position, pattern_length, "strict");
+                if (!pattern) {
+                    return NULL;
+                }
+                position += pattern_length + 1;
+                int flags_length = strlen(string + position);
+                int flags = 0;
+                int i;
+                for (i = 0; i < flags_length; i++) {
+                    if (string[position + i] == 'i') {
+                        flags |= 2;
+                    } else if (string[position + i] == 'l') {
+                        flags |= 4;
+                    } else if (string[position + i] == 'm') {
+                        flags |= 8;
+                    } else if (string[position + i] == 's') {
+                        flags |= 16;
+                    } else if (string[position + i] == 'u') {
+                        flags |= 32;
+                    } else if (string[position + i] == 'x') {
+                        flags |= 64;
+                    }
+                }
+                position += flags_length + 1;
+                value = PyObject_CallFunction(RECompile, "Oi", pattern, flags);
+                break;
+            }
+        case 12:
+            {
+                position += 4;
+                int collection_length = strlen(string + position);
+                PyObject* collection = PyUnicode_DecodeUTF8(string + position, collection_length, "strict");
+                if (!collection) {
+                    return NULL;
+                }
+                position += collection_length + 1;
+                char* shuffled = shuffle_oid(string + position);
+                PyObject* id = PyObject_CallFunction(ObjectId, "s#", shuffled, 12);
+                free(shuffled);
+                if (!id) {
+                    return NULL;
+                }
+                position += 12;
+                value = PyObject_CallFunctionObjArgs(DBRef, collection, id, NULL);
+                break;
+            }
+        case 16:
+            {
+                int i;
+                memcpy(&i, string + position, 4);
+                value = PyInt_FromLong(i);
+                if (!value) {
+                    return NULL;
+                }
+                position += 4;
+                break;
+            }
+        default:
+            printf("type: %d\n", type);
+            PyErr_SetString(CBSONError, "no c decoder for this type yet");
+            return NULL;
+        }
+        PyDict_SetItem(dict, name, value);
+    }
+    return dict;
+}
+
+static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* bson) {
+    if (!PyString_Check(bson)) {
+        PyErr_SetString(PyExc_TypeError, "argument to _bson_to_dict must be a string");
+        return NULL;
+    }
+    int size;
+    const char* string = PyString_AsString(bson);
+    if (!string) {
+        return NULL;
+    }
+    memcpy(&size, string, 4);
+
+    PyObject* elements = PySequence_GetSlice(bson, 4, size - 1);
+    if (!elements) {
+        return NULL;
+    }
+    PyObject* dict = _elements_to_dict(elements);
+    if (!dict) {
+        return NULL;
+    }
+    PyObject* remainder = PySequence_GetSlice(bson, size, PyString_Size(bson));
+    if (!remainder) {
+        return NULL;
+    }
+    return Py_BuildValue("OO", dict, remainder);
+}
+
 static PyMethodDef _CBSONMethods[] = {
-    {"_shuffle_oid", _cbson_shuffle_oid, METH_VARARGS,
-     "shuffle an ObjectId into proper byte order."},
-    {"_element_to_bson", _cbson_element_to_bson, METH_VARARGS,
-     "convert a key and value to its bson representation."},
     {"_dict_to_bson", _cbson_dict_to_bson, METH_O,
      "convert a dictionary to a string containing it's BSON representation."},
+    {"_bson_to_dict", _cbson_bson_to_dict, METH_O,
+     "convert a BSON string to a SON object."},
     {NULL, NULL, 0, NULL}
 };
 
@@ -416,4 +641,7 @@ PyMODINIT_FUNC init_cbson(void) {
 
     PyObject* dbref_module = PyImport_ImportModule("pymongo.dbref");
     DBRef = PyObject_GetAttrString(dbref_module, "DBRef");
+
+    PyObject* re_module = PyImport_ImportModule("re");
+    RECompile = PyObject_GetAttrString(re_module, "compile");
 }
