@@ -188,7 +188,7 @@ def _get_string(data):
     return _get_c_string(data[4:])
 
 def _get_object(data):
-    return _document_to_dict(data)
+    return _bson_to_dict(data)
 
 def _get_array(data):
     (obj, data) = _get_object(data)
@@ -282,15 +282,15 @@ def _elements_to_dict(data):
         result[key] = value
     return result
 
-def _document_to_dict(data):
+def _bson_to_dict(data):
     obj_size = struct.unpack("<i", data[:4])[0]
     elements = data[4:obj_size - 1]
     return (_elements_to_dict(elements), data[obj_size:])
+if _use_c:
+    _bson_to_dict = _cbson._bson_to_dict
 
 def _shuffle_oid(data):
     return data[7::-1] + data[:7:-1]
-if _use_c:
-    _shuffle_oid = _cbson._shuffle_oid
 
 _RE_TYPE = type(_valid_array_name)
 def _element_to_bson(key, value):
@@ -352,14 +352,6 @@ def _element_to_bson(key, value):
         ns = _make_c_string(value.collection())
         return "\x0C" + name + struct.pack("<i", len(ns)) + ns + _shuffle_oid(str(value.id()))
     raise InvalidDocument("cannot convert value of type %s to bson" % type(value))
-if _use_c:
-    _py_element_to_bson = _element_to_bson
-    _c_element_to_bson = _cbson._element_to_bson
-    def _element_to_bson(name, value):
-        try:
-            return _c_element_to_bson(name, value)
-        except _cbson.error:
-            return _py_element_to_bson(name, value)
 
 def _dict_to_bson(dict):
     try:
@@ -372,6 +364,24 @@ def _dict_to_bson(dict):
 if _use_c:
     _dict_to_bson = _cbson._dict_to_bson
 
+def _to_dicts(data):
+    """Convert binary data to sequence of SON objects.
+
+    Data must be concatenated strings of valid BSON data.
+
+    :Parameters:
+      - `data`: bson data
+    """
+    dicts = []
+    while len(data):
+        (son, data) = _bson_to_dict(data)
+        dicts.append(son)
+    return dicts
+
+def _to_dict(data):
+    (son, _) = _bson_to_dict(data)
+    return son
+
 def is_valid(bson):
     """Validate that the given string represents valid BSON data.
 
@@ -390,20 +400,6 @@ def is_valid(bson):
     except (AssertionError, InvalidBSON):
         return False
 
-def to_dicts(data):
-    """Convert binary data to sequence of SON objects.
-
-    Data must be concatenated strings of valid BSON data.
-
-    :Parameters:
-      - `data`: bson data
-    """
-    dicts = []
-    while len(data):
-        (son, data) = _document_to_dict(data)
-        dicts.append(son)
-    return dicts
-
 class BSON(str):
     """BSON data.
 
@@ -434,5 +430,5 @@ class BSON(str):
 
     def to_dict(self):
         """Get the dictionary representation of this data."""
-        (son, _) = _document_to_dict(self)
+        (son, _) = _bson_to_dict(self)
         return son
