@@ -106,6 +106,20 @@ class TestCollection(unittest.TestCase):
                               (u"ns", u"pymongo_test.test"),
                               (u"key", SON([(u"hello", 1)]))]))
 
+        db.test.drop_indexes()
+        db.test.create_index("hello", ASCENDING)
+        name = db.test.create_index("goodbye", DESCENDING)
+
+        self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"}).count(), 2)
+        self.assertEqual(name, "goodbye_-1")
+        db.test.drop_index([("goodbye", DESCENDING)])
+        self.assertEqual(db.system.indexes.find({"ns": u"pymongo_test.test"}).count(), 1)
+        self.assertEqual(db.system.indexes.find_one({"ns": u"pymongo_test.test"}),
+                         SON([(u"name", u"hello_1"),
+                              (u"ns", u"pymongo_test.test"),
+                              (u"key", SON([(u"hello", 1)]))]))
+
+
     def test_index_info(self):
         db = self.db
         db.test.drop_indexes()
@@ -116,10 +130,11 @@ class TestCollection(unittest.TestCase):
                          {"hello_1": [("hello", ASCENDING)]})
 
         db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)])
-        self.assertEqual(db.test.index_information(),
-                         {"hello_1": [("hello", ASCENDING)],
-                          "hello_-1_world_1": [("hello", DESCENDING),
-                                               ("world", ASCENDING)]})
+        self.assertEqual(db.test.index_information()["hello_1"], [("hello", ASCENDING)])
+        self.assertEqual(len(db.test.index_information()), 2)
+        self.assertTrue(("hello", DESCENDING) in db.test.index_information()["hello_-1_world_1"])
+        self.assertTrue(("world", ASCENDING) in db.test.index_information()["hello_-1_world_1"])
+        self.assertTrue(len(db.test.index_information()["hello_-1_world_1"]) == 2)
 
     def test_options(self):
         db = self.db
