@@ -16,15 +16,18 @@
 import unittest
 import types
 import random
+import sys
+sys.path[0:0] = [""]
 
-from test_connection import get_connection
-from pymongo.errors import InvalidOperation
+from pymongo.errors import InvalidOperation, OperationFailure
 from pymongo.cursor import Cursor
-from pymongo.database import Database, ASCENDING, DESCENDING
+from pymongo.database import Database
+from pymongo import ASCENDING, DESCENDING
+from test_connection import get_connection
 
 class TestCursor(unittest.TestCase):
     def setUp(self):
-        self.db = Database(get_connection(), "test")
+        self.db = Database(get_connection(), "pymongo_test")
 
     def test_explain(self):
         a = self.db.test.find()
@@ -43,12 +46,12 @@ class TestCursor(unittest.TestCase):
         for i in range(100):
             db.test.insert({"num": i, "foo": i})
 
-        self.assertEqual(db.test.find({"num": 17, "foo": 17}
-                                      ).hint([("num", ASCENDING)]
-                                             ).explain()["nscanned"], 100)
-        self.assertEqual(db.test.find({"num": 17, "foo": 17}
-                                      ).hint([("foo", ASCENDING)]
-                                             ).explain()["nscanned"], 100)
+        self.assertRaises(OperationFailure,
+                          db.test.find({"num": 17, "foo": 17}
+                                       ).hint([("num", ASCENDING)]).explain)
+        self.assertRaises(OperationFailure,
+                          db.test.find({"num": 17, "foo": 17}
+                                       ).hint([("foo", ASCENDING)]).explain)
 
         index = db.test.create_index("num", ASCENDING)
 
@@ -64,9 +67,9 @@ class TestCursor(unittest.TestCase):
         self.assertEqual(db.test.find({"num": 17, "foo": 17}
                                       ).hint([("num", ASCENDING)]
                                              ).explain()["nscanned"], 1)
-        self.assertEqual(db.test.find({"num": 17, "foo": 17}
-                                      ).hint([("foo", ASCENDING)]
-                                             ).explain()["nscanned"], 100)
+        self.assertRaises(OperationFailure,
+                          db.test.find({"num": 17, "foo": 17}
+                                       ).hint([("foo", ASCENDING)]).explain)
 
         a = db.test.find({"num": 17})
         a.hint(index)
@@ -241,5 +244,39 @@ class TestCursor(unittest.TestCase):
 
         self.assertEqual(0, db.test.acollectionthatdoesntexist.find().count())
 
+    def test_where(self):
+        db = self.db
+        db.test.remove({})
+
+        a = db.test.find()
+        self.assertRaises(TypeError, a.where, 5)
+        self.assertRaises(TypeError, a.where, None)
+        self.assertRaises(TypeError, a.where, {})
+
+        for i in range(10):
+            db.test.save({"x": i})
+
+        self.assertEqual(3, len(list(db.test.find().where('this.x < 3'))))
+        self.assertEqual(10, len(list(db.test.find())))
+
+        self.assertEqual(3, db.test.find().where('this.x < 3').count())
+        self.assertEqual(10, db.test.find().count())
+        self.assertEqual(3, db.test.find().where(u'this.x < 3').count())
+        self.assertEqual([0, 1, 2],
+                         [a["x"] for a in db.test.find().where('this.x < 3')])
+        self.assertEqual([],
+                         [a["x"] for a in db.test.find({"x": 5}).where('this.x < 3')])
+        self.assertEqual([5],
+                         [a["x"] for a in db.test.find({"x": 5}).where('this.x > 3')])
+
+        cursor = db.test.find().where('this.x < 3').where('this.x > 7')
+        self.assertEqual([8, 9], [a["x"] for a in cursor])
+
+        a = db.test.find()
+        b = a.where('this.x > 3')
+        for _ in a:
+            break
+        self.assertRaises(InvalidOperation, a.where, 'this.x < 3')
+
 if __name__ == "__main__":
     unittest.main()
